package com.qmth.cqb.genpaper.service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import com.qmth.cqb.utils.enums.PaperStructType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.qmth.cqb.base.dao.CourseRepo;
import com.qmth.cqb.base.model.Course;
import com.qmth.cqb.genpaper.context.DetailContext;
import com.qmth.cqb.genpaper.context.PaperContext;
import com.qmth.cqb.genpaper.context.UnitContext;
import com.qmth.cqb.genpaper.model.GenPaperDto;
import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.dao.PaperStructRepo;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.paper.model.PaperStruct;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.utils.BeanCopierUtil;
import com.qmth.cqb.utils.CombinationUtils;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.enums.PaperType;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;

@Service
public class GenPaperService {

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    PaperDetailRepo paperDetailRepo;
    
    @Autowired
    PaperDetailUnitRepo unitRepo;

    @Autowired
    PaperStructRepo paperStructRepo;

    @Autowired
    CourseRepo courseRepo;
    
    /**
     * 精确组卷，根据设定试卷结构组卷
     * 
     * @return
     */
    @SuppressWarnings("unchecked")
    public Map<String, Object> genPaper(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        LinkedList<Question> questions = new LinkedList<Question>();
        PaperStruct paperStruct = paperStructRepo.findOne(genPaperDto.getPaperStructId());
        PaperContext paperContext = new PaperContext(paperStruct, genPaperDto.getConditions());
        List<UnitContext> unitContexts = new LinkedList<UnitContext>();
        List<DetailContext> detailContexts = paperContext.getDetails();
        for (DetailContext dc : detailContexts) {
            unitContexts.addAll(dc.getUnits());
        }
        List<Paper> papers = (List<Paper>) paperRepo.findAll(genPaperDto.getPaperIds());
        for (Paper oldPaper : papers) {
            List<PaperDetailUnit> unitList = unitRepo.findByPaperOrderByNumber(oldPaper);
            for (PaperDetailUnit unit : unitList) {
                Question question = unit.getQuestion();
                question.setQuesName(unit.getPaperDetail().getName());
                questions.add(question);
            }

        }
        Collections.shuffle(questions);
        int maxLoopCount = questions.size() * 2;
        int selectCount = 0;
        int maxCount = unitContexts.size();
        int loopCount = 0;
        while (questions.size() > 0 && (selectCount < maxCount) && loopCount < maxLoopCount) {
            Question question = questions.removeFirst();
            boolean structTypeMatch = false;
            boolean selected = false;
            for (UnitContext uc : unitContexts) {
                if (checkQuesType(uc,question)) {
                    structTypeMatch = true;
                    if (uc.check(question)) {
                        uc.select(question);
                        selected = true;
                        selectCount++;
                        break;
                    }
                }
            }
            if (structTypeMatch && !selected) {
                questions.addLast(question);
            }
            loopCount++;
        }
        Set<Question> quesSet = paperContext.getSelectQues();
        if (quesSet.size() < unitContexts.size()) {
            msg = "匹配的题源不足，请核查";
            paperMsgMap.put("msg", msg);
            return paperMsgMap;
        }
        List<Object> list = paperContext.build(paperStruct);
        if (list != null && list.size() == 3) {
            Paper paper = (Paper) list.get(0);
            List<PaperDetail> paperDetails = (List<PaperDetail>) list.get(1);
            List<PaperDetailUnit> paperDetailunits = (List<PaperDetailUnit>) list.get(2);
            // 补全paper属性
            paper.setName(genPaperDto.getPaperName());
            paper.setCourseNo(genPaperDto.getCourseNo());
            paper.setCourseName(genPaperDto.getCourseName());
            Course course = courseRepo.findFirstByCodeAndOrgId(genPaperDto.getCourseNo(),genPaperDto.getOrgId());
            paper.setCourse(course);
            paper.setCreator(genPaperDto.getCreator());
            paper.setOrgId(genPaperDto.getOrgId());
            paper.setLevel(genPaperDto.getLevel());
            paper.setCreateTime(CommonUtils.getCurDateTime());
            paper.setUnitCount(this.getTotalQuesNum(paperDetailunits));
            paper.setPaperType(PaperType.GENERATE);
            // 数据入库
            paper = this.persistentPaper(PaperStructType.EXACT,paperDetailunits, paperDetails, paper);
            msg = "success";
            paperMsgMap.put("paper", paper);
            paperMsgMap.put("msg", msg);
        }
        return paperMsgMap;
    }

    /**
     * 校验来源大题名称
     * @param uc
     * @param question
     * @return
     */
    private boolean checkQuesType(UnitContext uc,Question question){
        List<String> quesNames = uc.getUnitStruct().getQuesNames();
        if(quesNames != null && quesNames.size() > 0){
            if(quesNames.contains(question.getQuesName())
                    && !uc.finish()
                    && (uc.getUnitStruct().getQuestionType() == question.getQuestionType())){
                return true;
            }
        }
        return false;
    }

    /**
     * 按分数在选定的试卷中随机抽取题,组成一套新的试卷
     */
    public Map<String, Object> genPaperByScore(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        Map<String, Double> paperMap = genPaperDto.getSimpleParams();
        List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();// 选中的小题
        if (paperMap != null && paperMap.size() > 0) {
            for (String paperId : paperMap.keySet()) {
                Double totalScore = paperMap.get(paperId);
                List<PaperDetailUnit> unitList = unitRepo.findByPaperOrderByNumber(paperRepo.findOne(paperId));
                Collections.shuffle(unitList);// 随机乱序之后再根据分数取题
                double sum = 0;
                for (PaperDetailUnit pdu : unitList) {
                    sum += pdu.getScore();
                    if (sum <= totalScore) {
                        selectedUnits.add(pdu);
                    } else {
                        break;
                    }
                }
            }
        }
        if (selectedUnits != null && selectedUnits.size() > 0) {
            Paper paper = this.constuctPaper(selectedUnits, genPaperDto);
            msg = "success";
            paperMsgMap.put("paper", paper);
            paperMsgMap.put("msg", msg);
            return paperMsgMap;
        } else {
            msg = "没有搜索到相应的试题，请检查题源试卷是否有题目";
            paperMsgMap.put("msg", msg);
            return paperMsgMap;
        }

    }

    /**
     * 按每套试卷需要抽取的题目数量随机抽取题,组成一套新的试卷
     * 
     * @return
     */
    public Map<String, Object> genPaperByQuestionNum(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        Map<String, Double> paperMap = genPaperDto.getSimpleParams();
        List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();// 选中的小题
        if (paperMap != null && paperMap.size() > 0) {
            for (String paperId : paperMap.keySet()) {
                int needQuesNum = paperMap.get(paperId).intValue();// 需要抽取的试题数量
                List<PaperDetailUnit> unitList = unitRepo.findByPaperOrderByNumber(paperRepo.findOne(paperId));
                Collections.shuffle(unitList);// 随机乱序之后再取题
                List<PaperDetailUnit> nestedUnits = new ArrayList<PaperDetailUnit>();// 已选的试题集合
                List<Integer> otherQuesNum = new ArrayList<Integer>();
                List<Integer> nestQums = new ArrayList<Integer>();
                if (this.getTotalQuesNum(unitList) >= needQuesNum) {
                    for (int i = 0; i < unitList.size(); i++) {
                        PaperDetailUnit pdu = unitList.get(i);
                        if (pdu.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                            if (pdu.getQuestion().getSubQuestions().size() <= needQuesNum) {
                                nestQums.add(pdu.getQuestion().getSubQuestions().size());
                                nestedUnits.add(pdu);
                            }
                        } else {
                            otherQuesNum.add(1);
                        }
                    }
                    List<Integer> canChooseNestNum = this.getNestNum(nestQums, otherQuesNum.size(), needQuesNum);
                    int size = canChooseNestNum.size();
                    int index = (int) (Math.random() * size);
                    int ranmdomNum = canChooseNestNum.get(index);// 取第一个
                    if (ranmdomNum == 0) {// 假如随机到0个套题
                        selectedUnits.addAll(this.getNeedUnits(null, unitList, needQuesNum));
                    } else {
                        List<Integer> sumList = new ArrayList<Integer>();
                        sumList = CombinationUtils.combiantion(nestQums, needQuesNum, ranmdomNum);
                        selectedUnits.addAll(this.getNeedUnits(sumList, unitList, needQuesNum));// 有套题情况
                    }
                } else {
                    msg = "题源不足，请检查题源";
                    paperMsgMap.put("msg", msg);
                    return paperMsgMap;
                }
            }
        }
        Paper paper = this.constuctPaper(selectedUnits, genPaperDto);
        msg = "success";
        paperMsgMap.put("paper", paper);
        paperMsgMap.put("msg", msg);
        return paperMsgMap;

    }

    /**
     * 构造大题
     * 
     * @param units
     * @param number
     * @param detailName
     *            大题名称
     * @return
     */
    public PaperDetail constuctPaperDeatil(List<PaperDetailUnit> units, int number, String detailName) {
        PaperDetail pd = new PaperDetail();
        if (units != null && units.size() > 0) {
            Double score = 0d;
            for (PaperDetailUnit pdu : units) {
                score += pdu.getScore() == null ? 0 : pdu.getScore();

            }
            pd.setId(UUID.randomUUID().toString());
            pd.setName(detailName);
            pd.setUnitCount(units.size());
            pd.setScore(score);
            pd.setNumber(number);
            pd.setCreateTime(CommonUtils.getCurDateTime());
            for (PaperDetailUnit pdu : units) {
                pdu.setPaperDetail(pd);
            }
        }
        return pd;

    }

    /**
     * 根据大题集合构造paper
     * 
     * @param details
     * @return
     */
    public Paper constuctPaperByPaperDetails(List<PaperDetail> details, GenPaperDto genPaperDto) {
        Paper paper = new Paper();
        if (details != null && details.size() > 0) {
            Double score = 0d;
            for (PaperDetail pd : details) {
                score += pd.getScore() == null ? 0 : pd.getScore();

            }
            paper.setId(UUID.randomUUID().toString());
            paper.setName(genPaperDto.getPaperName());
            paper.setCourseNo(genPaperDto.getCourseNo());
            paper.setCourseName(genPaperDto.getCourseName());
            Course course = courseRepo.findFirstByCodeAndOrgId(genPaperDto.getCourseNo(),genPaperDto.getOrgId());
            paper.setCourse(course);
            paper.setCreator(genPaperDto.getCreator());
            paper.setCreateTime(CommonUtils.getCurDateTime());
            paper.setPaperDetailCount(details.size());
            paper.setTotalScore(score);
            paper.setPaperType(PaperType.GENERATE);
            paper.setOrgId(genPaperDto.getOrgId());
            paper.setLevel(genPaperDto.getLevel());
        }
        return paper;

    }

    /**
     * 随机构造试卷,根据小题类型 没有定义试卷结构
     * 
     * @param selectedUnits
     * @param genPaperDto
     * @return
     */
    public Paper constuctPaper(List<PaperDetailUnit> selectedUnits, GenPaperDto genPaperDto) {
        Collections.sort(selectedUnits);// 根据大题题型ID排序
        List<PaperDetailUnit> saveUnits = BeanCopierUtil.copyPropertiesOfList(selectedUnits, PaperDetailUnit.class);
        List<QuesStructType> qsts = new ArrayList<QuesStructType>();
        for (int i = 0; i < saveUnits.size(); i++) {
            saveUnits.get(i).setId(UUID.randomUUID().toString());
            if (!qsts.contains(saveUnits.get(i).getQuestionType())) {
                qsts.add(saveUnits.get(i).getQuestionType());
            }
        }
        Map<QuesStructType, List<PaperDetailUnit>> unitMap = new HashMap<QuesStructType, List<PaperDetailUnit>>();
        Map<QuesStructType, Integer> qTypeIndexMap = new HashMap<QuesStructType, Integer>();
        for (int i = 1; i <= qsts.size(); i++) {
            List<PaperDetailUnit> tempUnits = new ArrayList<PaperDetailUnit>();
            for (int j = 0; j < saveUnits.size(); j++) {
                if (qsts.get(i - 1) == saveUnits.get(j).getQuestionType()) {
                    tempUnits.add(saveUnits.get(j));
                }
            }
            unitMap.put(qsts.get(i - 1), tempUnits);
            qTypeIndexMap.put(qsts.get(i - 1), i);
        }
        List<PaperDetail> details = new ArrayList<PaperDetail>();
        for (QuesStructType qType : unitMap.keySet()) {
            details.add(this.constuctPaperDeatil(unitMap.get(qType), qTypeIndexMap.get(qType), qType.getName()));
        }
        Paper paper = this.constuctPaperByPaperDetails(details, genPaperDto);
        paper.setUnitCount(this.getTotalQuesNum(saveUnits));// 设置小题数量，需统计subQuestion
        paper = this.persistentPaper(PaperStructType.SIMPLE,saveUnits, details, paper);
        return paper;

    }

    /**
     * 数据入库
     * 
     * @param details
     * @param selectedUnits
     * @param paper
     * @return
     */
    public Paper persistentPaper(PaperStructType paperStructType,
                                 List<PaperDetailUnit> selectedUnits,
                                 List<PaperDetail> details,
                                 Paper paper) {
        paper = paperRepo.save(paper);
        for (PaperDetail pd : details) {
            pd.setPaper(paper);
        }
        paperDetailRepo.save(details);
        for(PaperDetailUnit pdu:selectedUnits){
        	pdu.setPaper(paper);
        }
        unitRepo.save(selectedUnits);
        //给试卷中的小题设置number
        List<PaperDetail> paperDetails = paperDetailRepo.findByPaperOrderByNumber(paper);
        int index = 0;
        for(PaperDetail paperDetail:paperDetails){
        	List<PaperDetailUnit> paperDetailUnits = unitRepo.findByPaperDetailOrderByNumber(paperDetail);
        	for(int i = 0;i<paperDetailUnits.size();i++){
        		index++;
        		PaperDetailUnit paperDetailUnit = paperDetailUnits.get(i);
        		paperDetailUnit.setNumber(index);

                //重新计算套题小题分
                if(paperStructType != PaperStructType.SIMPLE){
                    Question question = paperDetailUnit.getQuestion();
                    if(question != null && question.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION){
                        List<Question> subQuestions = question.getSubQuestions();
                        List<Double> subScoreList = new ArrayList<>();
                        if(subQuestions != null && subQuestions.size() > 0){
                            int subQuesLen = subQuestions.size();
                            double subTotalScore = paperDetailUnit.getScore();
                            for(int j = 0;j < subQuesLen;j++){
                                double avgScore = subTotalScore / subQuestions.size();
                                //向下取整,包含0.5
                                double formatScore = CommonUtils.formatDoubleFloor(avgScore);
                                if(j == subQuesLen - 1){
                                    subScoreList.add(subTotalScore - formatScore * j);
                                }else{
                                    subScoreList.add(formatScore);
                                }
                            }
                            paperDetailUnit.setSubScoreListNew(subScoreList);
                        }
                    }
                }
        	}
        	unitRepo.save(paperDetailUnits);
        }
        return paper;
    }

    /**
     * 获取所试题数量
     * 
     * @param units
     * @return
     */
    public int getTotalQuesNum(List<PaperDetailUnit> units) {
        int totalQuesNum = units.size();
        int nestQuesNum = 0;
        for (PaperDetailUnit unit : units) {
            if (unit.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                nestQuesNum += unit.getQuestion().getSubQuestions().size() - 1;
            }
        }
        return totalQuesNum + nestQuesNum;
    }

    public List<PaperDetailUnit> getNeedUnits(List<Integer> sumList, List<PaperDetailUnit> unitList, int needQuesNum) {
        List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();// 选中的小题
        List<PaperDetailUnit> nestedUnits = new ArrayList<PaperDetailUnit>();// 套题集合
        List<PaperDetailUnit> otherUnits = new ArrayList<PaperDetailUnit>();// 其他类型的小题
        for (PaperDetailUnit pdu : unitList) {
            if (pdu.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                nestedUnits.add(pdu);
            } else {
                otherUnits.add(pdu);
            }
        }
        // 选的套题
        if (sumList != null && sumList.size() > 0) {
            for (Integer subNum : sumList) {
                Iterator<PaperDetailUnit> ite = nestedUnits.iterator();
                while (ite.hasNext()) {
                    PaperDetailUnit pdu = ite.next();
                    if (subNum == pdu.getQuestion().getSubQuestions().size()) {
                        selectedUnits.add(pdu);
                        ite.remove();
                        break;
                    }
                }

            }
        }
        int otherQuesNum = needQuesNum - this.getTotalQuesNum(selectedUnits);
        for (int i = 0; i < otherQuesNum; i++) {
            selectedUnits.add(otherUnits.get(i));
        }
        return selectedUnits;
    }

    public List<Integer> getNestNum(List<Integer> nestQums, int otherQuesNum, int needQuesNum) {
        Collections.sort(nestQums);
        List<Integer> chooseNestNum = new ArrayList<Integer>();
        if (needQuesNum <= otherQuesNum) {
            chooseNestNum.add(0);
        }
        for (int i = 1; i <= nestQums.size(); i++) {
            int min = 0;
            int max = 0;
            for (int j = 0; j < i; j++) {
                min += nestQums.get(j);
                max += nestQums.get(nestQums.size() - 1 - j);
            }
            if ((max + otherQuesNum >= needQuesNum && needQuesNum - max <= otherQuesNum && needQuesNum - max >= 0)
                    || (min + otherQuesNum >= needQuesNum && needQuesNum - min <= otherQuesNum
                            && needQuesNum - min >= 0)) {
                chooseNestNum.add(i);
            }
        }

        return chooseNestNum;

    }
}
