package com.qmth.cqb.paper.service.export;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import javax.xml.bind.JAXBElement;

import com.qmth.cqb.question.dao.QuesPkgPathRepo;

import main.java.com.UpYun;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.docx4j.XmlUtils;
import org.docx4j.jaxb.Context;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.wml.Body;
import org.docx4j.wml.P;
import org.docx4j.wml.R;
import org.docx4j.wml.Text;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Example;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;
import cn.com.qmth.examcloud.common.util.excel.ExcelWriter;

import com.google.gson.Gson;
import com.qmth.cqb.paper.dao.AudioTimeConfigRepo;
import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.dto.ObjectiveQuestionStructure;
import com.qmth.cqb.paper.dto.PaperDetailExp;
import com.qmth.cqb.paper.dto.PaperDetailUnitExp;
import com.qmth.cqb.paper.dto.PaperExp;
import com.qmth.cqb.paper.dto.SubjectiveQuestionStructure;
import com.qmth.cqb.paper.model.AudioTimeConfig;
import com.qmth.cqb.paper.model.ExamFile;
import com.qmth.cqb.paper.model.ExportStructure;
import com.qmth.cqb.paper.model.ExtractConfig;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.paper.model.QuestionTypeNum;
import com.qmth.cqb.paper.model.computerTestModel.Block;
import com.qmth.cqb.paper.model.computerTestModel.ComputerTestOption;
import com.qmth.cqb.paper.model.computerTestModel.ComputerTestPaper;
import com.qmth.cqb.paper.model.computerTestModel.ComputerTestPaperDetail;
import com.qmth.cqb.paper.model.computerTestModel.ComputerTestQuestion;
import com.qmth.cqb.paper.model.computerTestModel.Section;
import com.qmth.cqb.paper.model.computerTestModel.Sections;
import com.qmth.cqb.paper.service.ExamFileService;
import com.qmth.cqb.paper.service.PaperDetailService;
import com.qmth.cqb.paper.service.PaperService;
import com.qmth.cqb.question.dao.QuesRepo;
import com.qmth.cqb.question.model.QuesOption;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.question.model.QuestionAudio;
import com.qmth.cqb.question.service.QuestionAudioService;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.FileDisposeUtil;
import com.qmth.cqb.utils.enums.ExamFileType;
import com.qmth.cqb.utils.word.DocxProcessUtil;

import freemarker.template.Configuration;
import freemarker.template.Template;

/**
 * 
 * @author  	chenken
 * @date    	2017年7月15日 下午4:00:24
 * @company 	QMTH
 * @description 导出、上传文件service 父类
 */
public abstract class ExportPaperAbstractService {
	
	@Autowired
	protected PaperRepo paperRepo;

	@Autowired
	protected PaperService paperService;

	@Autowired
	protected PaperDetailRepo paperDetailRepo;

	@Autowired
	protected PaperDetailUnitRepo paperDetailUnitRepo;
	
	@Autowired
	protected ExamFileService examFileService;

	@Autowired
	protected PaperDetailService paperDetailService;
	
	@Autowired
	protected QuestionAudioService questionAudioService;
	
	@Autowired
	protected BuildComputerTestJsonService buildComputerTestJsonService;
	
	@Autowired
	protected InitPaperExpService initPaperExpService;
	
	@Autowired
	protected QuesRepo quesRepo;
	
	@Autowired
	protected AudioTimeConfigRepo audioTimeConfigRepo;

	@Autowired
	protected QuesPkgPathRepo quesPkgPathRepo;

	public static final String FILL_BLANK_QUESTION_FLAG = "###";
	
	public static final String TEMP_FILE_EXP = "docxExport/";
	
	/**
	 * Word文件后缀
	 */
	public static final String DOCX_SUFFIX = ".docx";
	/**
	 * Excel文件后缀
	 */
	public static final String EXCEL_SUFFIX = ".xlsx";
	/**
	 * ZIP文件后缀
	 */
	public static final String ZIP_SUFFIX = ".zip";
	
	public static final String ENCODING = "UTF-8";
	/**
	 * 下划线
	 */
	private static final String UNDERSCORE = "_";

	//陕西师范模板
	protected static Template SXSF_TEMPLATE_PAPER;
	protected static Template SXSF_TEMPLATE_ANSWER;
	
	//电子科大模板
	protected static Template DZKD_TEMPLATE_PAPER;
	protected static Template DZKD_TEMPLATE_ANSWER;
	
	//华中科技大学模板
	protected static Template HZKJ_TEMPLATE_PAPER;
	protected static Template HZKJ_TEMPLATE_ANSWER;
	
	//山东大学
	protected static Template SDDX_TEMPLATE_PAPER;
	protected static Template SDDX_TEMPLATE_ANSWER;
	
	//天津大学
	protected static Template TJDX_TEMPLATE_PAPER;
	protected static Template TJDX_TEMPLATE_ANSWER;
	
	//石油大学模板
	protected static Template SYDX_TEMPLATE_PAPER;
	protected static Template SYDX_TEMPLATE_ANSWER;
	
	//西安交大模板
	protected static Template XAJD_TEMPLATE_PAPER;
	protected static Template XAJD_TEMPLATE_ANSWER;

	@Value("${upyun.bucketName}")
	protected String bucketName;
	
	@Value("${upyun.userName}")
	protected String userName;
	
	@Value("${upyun.password}")
	protected String password;
	
	@Value("${upyun.uploadUrl}")
	protected String uploadUrl;
	
	@Value("${upyun.downloadUrl}")
	protected String downloadUrl;
	
	public static Configuration CONFIGURATION;
	
	static {
		try {
			CONFIGURATION = new Configuration(Configuration.VERSION_2_3_25);
			// 设置编码
			CONFIGURATION.setDefaultEncoding(ENCODING);
			// 设置ftl模板路径
			CONFIGURATION.setClassForTemplateLoading(DocxProcessUtil.class, "/");
			
			CONFIGURATION = new Configuration(Configuration.VERSION_2_3_25);
			// 设置编码
			CONFIGURATION.setDefaultEncoding(ENCODING);
			// 设置ftl模板路径
			CONFIGURATION.setClassForTemplateLoading(DocxProcessUtil.class, "/");
			
			SXSF_TEMPLATE_PAPER = CONFIGURATION.getTemplate("sxsf_paper_template.ftl", ENCODING);
			SXSF_TEMPLATE_ANSWER = CONFIGURATION.getTemplate("sxsf_answer_template.ftl", ENCODING);
			
			DZKD_TEMPLATE_PAPER = CONFIGURATION.getTemplate("dzkd_paper_template.ftl", ENCODING);
			DZKD_TEMPLATE_ANSWER = CONFIGURATION.getTemplate("dzkd_answer_template.ftl", ENCODING);
			
			HZKJ_TEMPLATE_PAPER = CONFIGURATION.getTemplate("hzkj_paper_template.ftl", ENCODING);
			HZKJ_TEMPLATE_ANSWER = CONFIGURATION.getTemplate("hzkj_answer_template.ftl", ENCODING);
			
			SDDX_TEMPLATE_PAPER = CONFIGURATION.getTemplate("sddx_paper_template.ftl", ENCODING);
			SDDX_TEMPLATE_ANSWER = CONFIGURATION.getTemplate("sddx_answer_template.ftl", ENCODING);
			
			TJDX_TEMPLATE_PAPER = CONFIGURATION.getTemplate("tjdx_paper_template.ftl", ENCODING);
			TJDX_TEMPLATE_ANSWER = CONFIGURATION.getTemplate("tjdx_answer_template.ftl", ENCODING);
			
			SYDX_TEMPLATE_PAPER = CONFIGURATION.getTemplate("sydx_paper_template.ftl", ENCODING);
			SYDX_TEMPLATE_ANSWER = CONFIGURATION.getTemplate("sydx_answer_template.ftl", ENCODING);
			
			XAJD_TEMPLATE_PAPER = CONFIGURATION.getTemplate("xajd_paper_template.ftl", ENCODING);
			XAJD_TEMPLATE_ANSWER = CONFIGURATION.getTemplate("xajd_answer_template.ftl", ENCODING);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 下载试卷
	 * @param paperId
	 * @param zipFileName
	 * @throws Exception
	 */
	public abstract void downloadPaper(String paperId,String zipFileName)throws Exception;
	/**
	 * 下载答案
	 * @param paperId
	 * @param zipFileName
	 * @throws Exception
	 */
	public abstract void downloadPaperAnswer(String paperId,String zipFileName)throws Exception;
	/**
	 * 上传试卷相关文件
	 * @param extractConfig
	 * @param paperId
	 * @param exportStructure
	 * @param accessUser
	 * @throws Exception
	 */
    public abstract void uploadFile(ExtractConfig extractConfig,String paperId,ExportStructure exportStructure,AccessUser accessUser)  throws Exception;
    
    public PaperExp initPaperExp(String paperId) throws Exception{
    	PaperExp paperExp = initPaperExpService.initPaperExp(paperId);
    	//给新的大题下所有小题排序
    	sortPaperDetailUnitExps(paperExp.getPaperDetails());
    	//如果每个小题分数不一样，题干后面添加分数
    	appendScoreToQuestionBody(paperExp.getPaperDetails());
    	//1.设置题号 2.给小题选项进行排序 3.设置选项号 4.替换题干中的##为____
    	setUnitExpNumber(paperExp.getPaperDetails());
    	//设置大题标题
    	setExpDtoTitle(paperExp.getPaperDetails());
    	return paperExp;
    }
    
    /**
     * 设置选项号
     * @param optionWordMl
     * @param num
     * @return
     * @throws Exception
     */
    public String setOptionNum(String optionWordMl,String num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + optionWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for(Object pObj:pList){
            if(index > 0){
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            R run = new R();
            Text text = new Text();
            text.setValue(num+". ");
            run.getContent().add(text);
            pContent.add(0,run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }
    
    /**
   	 * 将数字1,2,3,4转化成A,B,C,D
   	 * @param number
   	 * @return
   	 */
       public String getOptionNum(int number){
           char optionNum = (char)(65 + number);
           return String.valueOf(optionNum);
       }
    
       /**
        * 替换填空
        * 将###替换为下划线_______
        * @param wordMl
        * @param num
        * @return
        * @throws Exception
        */
   	public String replaceQuesBlank(String wordMl,int num) throws Exception {
           String tmpStr = DocxProcessUtil.BODY_HEADER + wordMl + DocxProcessUtil.BODY_TAIL;
           Body body = (Body)XmlUtils.unmarshalString(tmpStr,Context.jc,Body.class);
           List<Object> pList = body.getContent();
           int index = 0;
           int cur = 0;
           Map<Integer,String> curMap = new HashMap<Integer,String>();
           for(Object pObj:pList){
               if(pObj.getClass().equals(P.class)){
                   List<Object> pContent = ((P)pObj).getContent();
                   for(Object rObj:pContent){
                       if(rObj.getClass().equals(R.class)){
                           List<Object> rContent = ((R)rObj).getContent();
                           for(Object tObj:rContent){
                               if (tObj instanceof JAXBElement){
                            	   tObj = ((JAXBElement<?>) tObj).getValue();
                               }
                               if(tObj.getClass().equals(Text.class)){
                                   Text text = (Text)tObj;
                                   String str = text.getValue();
                                   //1
                                   if("##".equals(str.trim())){
                                   	str = "___";
                                       text.setValue(str);
                                   }
                                   //2
                                   str = str.replaceAll("###","______");
                                   text.setValue(str);
                                   //3
                                   Pattern pattern = Pattern.compile("##\\d{1,}##");
	                           		Matcher m = pattern.matcher(str);
	                               	while(m.find()){
	                               		int curNum = num + index;
	                                       String a = m.group();
	                                       str = str.replaceAll(a, "___"+(curNum)+"___");
	                                       text.setValue(str);
	                                       index++;
	                               	}
	                               	//4
	                               /*if(str.startsWith("#") || str.equals("___")){
	                                   curMap.put(cur,str);
	                                   text.setValue("");
	                               }*/
	                               //5
	                               if(str.matches("^\\d{1,}$")){
	                                   String preStr = curMap.get(cur - 1);
	                                   int curNum = num + index;
	                                   if(!StringUtils.isEmpty(preStr) && preStr.startsWith("#")){
	                                       text.setValue("___"+(curNum)+"___");
	                                   }
	                                   index++;
	                               }
	                               cur++;
                               }
                           }
                       }
                   }
               }
           }
           StringBuffer pWordMl = new StringBuffer();
           for(Object pObj:pList){
               if(pObj instanceof P){
                   pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
               }
           }
           return pWordMl.toString();
       }
	
	/**
     * 设置题号
     * @param quesBodyWordMl
     * @param num
     * @return
     * @throws Exception
     */
    public String setSubQuesNum(String quesBodyWordMl,int num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + quesBodyWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for(Object pObj:pList){
            if(index > 0){
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            R run = new R();
            Text text = new Text();
            text.setValue(num+". ");
            run.getContent().add(text);
            pContent.add(0,run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }
    
    /**
     * 1.设置题号
	 * 2.给小题选项进行排序
	 * 3.设置选项号
	 * 4.替换题干中的##为____
	 * @param paperDetails
	 * @throws Exception
	 */
	public void setUnitExpNumber(List<PaperDetailExp> paperDetails) throws Exception {
		for (PaperDetailExp paperDetail : paperDetails) {
			List<PaperDetailUnitExp> paperDetailUnitExpList = paperDetail.getPaperDetailUnits();
			for (PaperDetailUnitExp paperDetailUnit:paperDetailUnitExpList) {
				List<QuesOption> optionList = paperDetailUnit.getQuestion().getQuesOptions();
				if (optionList != null && optionList.size() > 0) {
					int index = 0;
					for (QuesOption quesOption : optionList) {
						quesOption.setOptionBodyWord(setOptionNum(quesOption.getOptionBodyWord(),getOptionNum(index)));
						index++;
					}
				}
				List<Question> subQuesList = paperDetailUnit.getQuestion().getSubQuestions();
				Question question = paperDetailUnit.getQuestion();
				if (subQuesList != null && subQuesList.size() > 0) {
					//套题主题干
					question.setQuesBodyWord(replaceQuesBlank(question.getQuesBodyWord(),subQuesList.get(0).getNumber()));
					for (Question subQues : subQuesList) {
						//处理子题题干，答案
						subQues.setQuesBodyWord(setSubQuesNum(subQues.getQuesBodyWord(),subQues.getNumber()));
						subQues.setQuesAnswerWord(setSubQuesNum(subQues.getQuesAnswerWord(),subQues.getNumber()));
						subQues.setQuesBodyWord(replaceQuesBlank(subQues.getQuesBodyWord(),subQues.getNumber()));
						List<QuesOption> subOptionList = subQues.getQuesOptions();
						if (subOptionList != null && subOptionList.size() > 0) {
							int sub_index = 0;
							for (QuesOption quesOption : subOptionList) {
								quesOption.setOptionBodyWord(setOptionNum(quesOption.getOptionBodyWord(),getOptionNum(sub_index)));
								sub_index++;
							}
						}
					}
				} else {
					//处理非套题题干、答案
					question.setQuesBodyWord(setSubQuesNum(question.getQuesBodyWord(), paperDetailUnit.getNumber()));
					question.setQuesAnswerWord(setSubQuesNum(question.getQuesAnswerWord(),paperDetailUnit.getNumber()));
					question.setQuesBodyWord(replaceQuesBlank(question.getQuesBodyWord(),paperDetailUnit.getNumber()));
				}
			}
		}
	}
	
    /**
     * 在题干上设置分数
     * @throws Exception 
     */
    public String appendScoreToQuestionBody(String quesBodyWordMl,double score) throws Exception{
    	String totalScore = BigDecimal.valueOf(score).stripTrailingZeros().toPlainString();
    	String scores = deletePoint(totalScore);
    	String tmpStr = DocxProcessUtil.BODY_HEADER + quesBodyWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for(Object pObj:pList){
            if(index == pList.size()-1){
				P p = (P) pObj;
				List<Object> pContent = p.getContent();
				R run = new R();
				Text text = new Text();
				text.setValue("("+scores+"分)");
				run.getContent().add(text);
				pContent.add(run);

            }
			index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }
    
    //去掉分数后面的小数点
  	private String deletePoint(String str) {
  		if(str.indexOf(".0")>-1){
  			return str.replace(".0", "");
  		}
  		return str;
  	}
  	
    /**
     * 当大题下小题分数不是全部一样时，在小题题干后面加上小题的分数
     * @param details
     * @throws Exception
     */
    public void appendScoreToQuestionBody(List<PaperDetailExp> details) throws Exception{
    	for(PaperDetailExp paperDetailExp:details){
    		if(checkPaperDetailUnitScore(paperDetailExp)==null){
    			for(PaperDetailUnitExp paperDetailUnit:paperDetailExp.getPaperDetailUnits()){
    				//判断小题是否为套题
    				if(paperDetailUnit.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION){
    					List<Question> subQuestions = paperDetailUnit.getQuestion().getSubQuestions();
    					for(int i = 0;i < subQuestions.size();i++){
    						Question subQuestion = subQuestions.get(i);
							double subScore = paperDetailUnit.getSubScoreList().get(i);
    						String questionBodyWord = appendScoreToQuestionBody(subQuestion.getQuesBodyWord(),subScore);
    						subQuestion.setQuesBodyWord(questionBodyWord);
    					}
    				}else {
    					Question question = paperDetailUnit.getQuestion();
        				String questionBodyWord = appendScoreToQuestionBody(question.getQuesBodyWord(),paperDetailUnit.getScore());
        				question.setQuesBodyWord(questionBodyWord);
					}
    			}
    		}
    	}
    }
    
    /**
     * 小题排序
     * @describle 大题排序完成之后，放在一个集合里面。然后对相同类型的大题分组，生成不同的结合，在对小题进行
     * @param paperDetailExps
     */
    public void sortPaperDetailUnitExps(List<PaperDetailExp> paperDetailExps) {
    	Map<Long,List<PaperDetailUnitExp>> map = new HashMap<Long, List<PaperDetailUnitExp>>();
    	//循环所有大题,将同类型的小题放到同一个集合下面
    	int index = 0;
    	for(PaperDetailExp paperDetailExp:paperDetailExps){
    		List<PaperDetailUnitExp> paperDetailUnitExps = null;
    		//单选，多选，判断放在同一集合下排序
    		if(paperDetailExp.getSortNumber()<4L){
    			//通过map的key去取value，如果没有，就添加
        		if(map.get(paperDetailExp.getSortNumber()) == null){
        			paperDetailUnitExps = new ArrayList<PaperDetailUnitExp>();
        		}else {
    				paperDetailUnitExps = map.get(paperDetailExp.getSortNumber());
    			}
        		paperDetailUnitExps.addAll(paperDetailExp.getPaperDetailUnits());
        		map.put(paperDetailExp.getSortNumber(), paperDetailUnitExps);
    		}else {
    			paperDetailUnitExps = new ArrayList<PaperDetailUnitExp>();
    			paperDetailUnitExps.addAll(paperDetailExp.getPaperDetailUnits());
    			map.put(paperDetailExp.getSortNumber() + index, paperDetailUnitExps);
    			index++;
			}
    	}
    	//循环所有小题，给小题排序
    	for(Long sortNumber:map.keySet()){
    		List<PaperDetailUnitExp> paperDetailUnitExps = map.get(sortNumber);
    		int number = 1;
    		if(paperDetailUnitExps != null && paperDetailUnitExps.size()>0){
    			for(int i = 0;i<paperDetailUnitExps.size();i++){
    				PaperDetailUnitExp paperDetailUnitExp = paperDetailUnitExps.get(i);
    				if(paperDetailUnitExp.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION){
    					List<Question> subQuestions = paperDetailUnitExp.getQuestion().getSubQuestions();
    					for(int j = 0;j<subQuestions.size();j++){
    						subQuestions.get(j).setNumber(number+j);
    					}
    					number += subQuestions.size();
    				}else {
    					paperDetailUnitExp.setNumber(number);
    					number++;
					}
        		}
    		}
    	}
	}

    
    /**
   	 * 设置大题标题
   	 * @param paperDetailExps
   	 */
   	public void setExpDtoTitle(List<PaperDetailExp> paperDetailExps) {
   		for(PaperDetailExp paperDetailExp:paperDetailExps){
   			String title = "";
   			String titleDetail = null;
   			String totalScore = BigDecimal.valueOf(paperDetailExp.getScore()).stripTrailingZeros().toPlainString();
   			QuesStructType type = paperDetailExp.getPaperDetailUnits().get(0).getQuestionType();
   			String questionScore = checkPaperDetailUnitScore(paperDetailExp);
   			if(questionScore!=null){
   				questionScore = deletePoint(questionScore);
   			}
   			String scoreString = questionScore==null?"":"每小题"+questionScore+"分，";
   			if (type == QuesStructType.SINGLE_ANSWER_QUESTION) {
   				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
   						+ scoreString + "共" + totalScore + "分）";
   				titleDetail = "在每小题列出的备选项中只有一个符合题目要求的，请将其选出并将“答题卡”的相应代码涂黑，错涂、多涂或未涂均无分";
   			} else if (type == QuesStructType.MULTIPLE_ANSWER_QUESTION) {
   				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
   						+ scoreString + "共" + totalScore+ "分）";
   				titleDetail = "在每小题列出的备选项中有一个或多个符合题目要求的，请将符合项选出并将“答题卡”的相应代码涂黑";
   			} else if (type == QuesStructType.BOOL_ANSWER_QUESTION) {
   				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
   						+ scoreString + "共" + totalScore+ "分。"
   						+ "正确的填涂√、错误填涂×。错涂、多涂或未涂均无分）";
   			} else {
   				title = "（本大题共"+paperDetailExp.getUnitCount()+"小题，"+scoreString+"共"+totalScore+"分）";
   			}
   			paperDetailExp.setTitle(title);
   			paperDetailExp.setTitleDetail(titleDetail);
   		}
   	}

   	/**
	 * 校验大题下的小题分数是不是一样
	 * @param paperDetailExp
	 * @return  如果分数一样，返回分数
	 * 			如果分数不一样，返回 null
	 */
	public String checkPaperDetailUnitScore(PaperDetailExp paperDetailExp) {
		List<PaperDetailUnitExp> paperDetailUnitExps = paperDetailExp.getPaperDetailUnits();
		Set<Double> scoreSet = new HashSet<Double>();
		for (PaperDetailUnitExp unitExp : paperDetailUnitExps) {
			//判断套路
			if(unitExp.getQuestionType()==QuesStructType.NESTED_ANSWER_QUESTION){
//				List<Question> subQuestions = unitExp.getQuestion().getSubQuestions();
//				for(Question subQuestion:subQuestions){
//					scoreSet.add(subQuestion.getScore());
//				}
				List<Double> subScoreList =  unitExp.getSubScoreList();
				for(Double score:subScoreList){
					scoreSet.add(score);
				}
			}else{
				scoreSet.add(unitExp.getScore());
			}
		}
		if(scoreSet.size() == 1){
			return scoreSet.iterator().next()+"";
		}
		return null;
	}

   	
    /**
     * 创建机考文件，并打包上传至又拍云
     * @param extractConfig
     * @param accessUser
     * @throws IOException 
     */
    protected void uploadComputerTestFile(ExtractConfig extractConfig,AccessUser accessUser) throws IOException{
    	List<ComputerTestPaper>  computerTestPaperList = buildComputerTestJsonService.buildComputerTestPapers(extractConfig);
    	String currentTimeStr = CommonUtils.getCurNum();
    	for(int i = 0;i<computerTestPaperList.size();i++){
    		ComputerTestPaper computerTestPaper = computerTestPaperList.get(i);
    		String jsonDirectoryName = currentTimeStr+UNDERSCORE+ExamFileType.COMPUTERTEST_PACKAGE.name()+UNDERSCORE+i;
    		String jsonDirectoryPath = TEMP_FILE_EXP + File.separator + jsonDirectoryName;
    		//新建文件夹
    		File dirFile = new File(jsonDirectoryPath);
    		if(!dirFile.exists()){
    			dirFile.mkdirs();
    		}
    		//下载试卷中的音频文件到jsonDirectoryPath文件夹中
			downloadQuestionAudio(computerTestPaper,jsonDirectoryPath,extractConfig.getExamId()+"");
    		//将computerTestPaper对象生成JSON文件存放在jsonDirectoryPath中
    		makeComputerTestPaperToJsonFile(extractConfig,computerTestPaper,jsonDirectoryPath);
			//将文件夹打包成zip压缩包放在docxExport下
			FileDisposeUtil.fileToZip(jsonDirectoryPath,"docxExport",jsonDirectoryName);
			//上传zip压缩包到又拍云
			File zipFile = new File(TEMP_FILE_EXP+jsonDirectoryName+ZIP_SUFFIX);
			String zipUpyunFilePath = uploadUrl+extractConfig.getOrgId()+"/"+jsonDirectoryName+ZIP_SUFFIX;
			UpYun upyun = new UpYun(bucketName,userName,password);
			upyun.writeFile(zipUpyunFilePath,zipFile,true);
			//保存数据库记录
			examFileService.saveExamFile(new ExamFile(computerTestPaper,extractConfig,zipUpyunFilePath,ExamFileType.COMPUTERTEST_PACKAGE,ZIP_SUFFIX),accessUser);
			//删除本地生成的文件
			zipFile.delete();
			FileUtils.deleteQuietly(dirFile);
    	}
    }
    
    /**
     * 将computerTestPaper对象生成JSON文件存放在jsonDirectoryPath中
     * @param extractConfig
     * @param computerTestPaper
     * @param jsonDirectoryPath
     */
    private void makeComputerTestPaperToJsonFile(ExtractConfig extractConfig,ComputerTestPaper computerTestPaper,String jsonDirectoryPath){
		//创建新的JSON文件
		File file = new File(jsonDirectoryPath+File.separator+extractConfig.getCourseCode()+".json");
		//将对象转成 json对象
		Gson gson = new Gson();
		String strJSON = gson.toJson(computerTestPaper);
		//生成文件流写入JSON文件
		FileOutputStream outputStream;
		try {
			outputStream = new FileOutputStream(file);
			byte b[] = strJSON.getBytes();
			outputStream.write(b);
			outputStream.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
    
    /**
     * 下载题目音频文件
     * @param computerTestPaper
     */
    private void downloadQuestionAudio(ComputerTestPaper computerTestPaper,String jsonDirectoryPath,String examId){
    	List<ComputerTestPaperDetail> details = computerTestPaper.getDetails();
		for(ComputerTestPaperDetail detail:details){
			List<ComputerTestQuestion> questions = detail.getQuestions();
			for(ComputerTestQuestion computerTestQuestion:questions){
				getBodyAndOptionAudioFile(computerTestQuestion,computerTestQuestion.getBody(),jsonDirectoryPath,computerTestPaper,examId);
				List<ComputerTestOption> options = computerTestQuestion.getOptions();
				for(ComputerTestOption option:options){
					getBodyAndOptionAudioFile(computerTestQuestion,option.getBody(),jsonDirectoryPath,computerTestPaper,examId);
				}
				//子题音频处理
				//List<ComputerTestQuestion> subQuestions = question.getSubQuestions();
			}
		}
    }
    
    private void getBodyAndOptionAudioFile(ComputerTestQuestion computerTestQuestion,
										   Sections bodySections,
										   String jsonDirectoryPath,
										   ComputerTestPaper computerTestPaper,
										   String examId){
    	List<Section> sectionList = bodySections.getSections();
		for(Section section:sectionList){
			List<Block> blocks = section.getBlocks();
			for(Block block:blocks){
				if("audio".equals(block.getType())){
					computerTestPaper.setHasVideo(1);
					PaperDetailUnit paperDetailUnit = paperDetailUnitRepo.findById(computerTestQuestion.getId());
					QuestionAudio audio = questionAudioService.findAudioById(block.getValue());
					block.setValue(block.getValue()+"."+audio.getFileSuffixes());
					if(audio!=null){
						AudioTimeConfig audioTime = new AudioTimeConfig();
						audioTime.setCourseCode(computerTestPaper.getCourseCode());
						audioTime.setExamId(examId);
						audioTime.setGroupCode(computerTestPaper.getGroupCode());
						audioTime.setPaperDetailUnit(paperDetailUnit);
						AudioTimeConfig audioTimeConfig = audioTimeConfigRepo.findOne(Example.of(audioTime));
						if(audioTimeConfig!=null){
							block.setPlayTime(audioTimeConfig.getPlayTime());
						}
						FileDisposeUtil.saveUrlAs(downloadUrl+audio.getFileUrl(),jsonDirectoryPath+File.separator+block.getValue());
					}
				}
			}
		}
    }
    
	/**
     * 生成试卷或答案Word，上传至又拍云
     * @param paperExp
     * @param extractConfig
     * @param accessUser
     * @param currNum
     * @param template
	 * @param examFileType
     */
    protected void uploadPaperOrAnswerFile(PaperExp paperExp,
										   ExtractConfig extractConfig,
										   AccessUser accessUser,
										   String currNum,
										   Template template,
										   ExamFileType examFileType){
    	String paperfileName = currNum+examFileType.name()+DOCX_SUFFIX;
    	try {
			DocxProcessUtil.exportWord(paperExp,paperfileName,template);
			DocxProcessUtil.processImage(paperfileName,getPkgList(paperExp.getId()));
			File file  = new File(TEMP_FILE_EXP+paperfileName);
			String paperFilePath = uploadUrl+extractConfig.getOrgId()+"/"+paperfileName;
            UpYun upyun = new UpYun(bucketName,userName,password);
			upyun.writeFile(paperFilePath,file,true);
			examFileService.saveExamFile(new ExamFile(paperExp.getId(),extractConfig,paperFilePath,examFileType,DOCX_SUFFIX),accessUser);
			file.delete();
		} catch (Exception e) {
			e.printStackTrace();
		}
    }
    
    /**
     * 生成试卷结构，上传至又拍云
     */
    protected void uploadPaperStructure(PaperExp paperExp,ExtractConfig extractConfig,AccessUser accessUser,String currNum,List<QuestionTypeNum> questionTypeNums) {
    	exportObjectiveQuestionStructures(paperExp,extractConfig,accessUser,currNum,questionTypeNums);
    	exportSubjectiveQuestionStructures(paperExp,extractConfig,accessUser,currNum);
	}
    
    /**
     * 获得客观题导出结构，并上传至又拍云
     * @param paperExp
     * @return
     */
    protected void exportObjectiveQuestionStructures(PaperExp paperExp,ExtractConfig extractConfig,AccessUser accessUser,String currNum,List<QuestionTypeNum> questionTypeNums){
    	String objectiveFilename = currNum+ExamFileType.PAPER_STRUCTURE_OBJECTIVE.name()+EXCEL_SUFFIX;
    	//得到试卷中的客观大题
    	List<PaperDetailExp> objectiveDetails = getAllObjectiveDetails(paperExp);
    	//根据试卷结构导出设置中的数量补齐客观题
    	List<PaperDetailExp> paperDetailExps = fillObjectiveQuestions(objectiveDetails,questionTypeNums);
    	List<ObjectiveQuestionStructure> objectiveQuestionStructureList = new ArrayList<ObjectiveQuestionStructure>();
    	
    	String paperType = findPaperType(extractConfig,paperExp.getId());
    	for(PaperDetailExp paperDetailExp:paperDetailExps){
    		for(PaperDetailUnitExp unit:paperDetailExp.getPaperDetailUnits()){
    			objectiveQuestionStructureList.add(new ObjectiveQuestionStructure(paperExp,paperDetailExp,unit,paperType));
    		}
    	}
    	ExcelWriter objectiveExcelExporter = new ExcelWriter(ObjectiveQuestionStructure.class); 
		try {
			File file = new File(TEMP_FILE_EXP+objectiveFilename);
			FileOutputStream out = new FileOutputStream(file);
			objectiveExcelExporter.write(objectiveFilename,objectiveQuestionStructureList,out);
			String objectiveFilePath = uploadUrl+extractConfig.getOrgId()+"/"+objectiveFilename;
            UpYun upyun = new UpYun(bucketName,userName,password);
			upyun.writeFile(objectiveFilePath,file,true);
			examFileService.saveExamFile(new ExamFile(paperExp.getId(),extractConfig,objectiveFilePath,ExamFileType.PAPER_STRUCTURE_OBJECTIVE,EXCEL_SUFFIX),accessUser);
			file.delete();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
    

   	/**
     * 获得主观题导出结构，并上传至又拍云
     * @param paperExp
     * @return
     */
    protected void exportSubjectiveQuestionStructures(PaperExp paperExp,ExtractConfig extractConfig,AccessUser accessUser,String currNum){
    	String subjectiveFileName = currNum+ExamFileType.PAPER_STRUCTURE_SUBJECTIVE.name()+EXCEL_SUFFIX;
    	List<PaperDetailExp> subjectiveDetails = getAllSubjectiveDetails(paperExp);
    	List<SubjectiveQuestionStructure> subjectiveQuestionStructureList = new ArrayList<SubjectiveQuestionStructure>();
    	String paperType = findPaperType(extractConfig,paperExp.getId());
    	for(PaperDetailExp paperDetailExp:subjectiveDetails){
    		for(PaperDetailUnitExp unit:paperDetailExp.getPaperDetailUnits()){
    			subjectiveQuestionStructureList.add(new SubjectiveQuestionStructure(paperExp,paperDetailExp,unit,paperType));
    		}
    	}
    	ExcelWriter subjectiveExcelExporter = new ExcelWriter(SubjectiveQuestionStructure.class); 
		try {
			File file = new File(TEMP_FILE_EXP+subjectiveFileName);
			FileOutputStream out = new FileOutputStream(file);
			subjectiveExcelExporter.write(subjectiveFileName,subjectiveQuestionStructureList,out);
			String subjectiveFilePath = uploadUrl+extractConfig.getOrgId()+"/"+subjectiveFileName;
            UpYun upyun = new UpYun(bucketName,userName,password);
			upyun.writeFile(subjectiveFilePath,file,true);
			examFileService.saveExamFile(new ExamFile(paperExp.getId(),extractConfig,subjectiveFilePath,ExamFileType.PAPER_STRUCTURE_SUBJECTIVE,EXCEL_SUFFIX),accessUser);
			file.delete();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
    
    /**
     * 根据试卷结构导出设置中的数量补齐客观题
     * @param objectiveDetails
     * @param questionTypeNums
     */
    public List<PaperDetailExp> fillObjectiveQuestions(List<PaperDetailExp> objectiveDetails,List<QuestionTypeNum> questionTypeNums) {
    	//1.得到所有客观题大题的类型
    	Set<QuesStructType> types = new HashSet<QuesStructType>();
    	for(PaperDetailExp paperDetailExp:objectiveDetails){
    		types.add(QuesStructType.getQuesStructTypeById(paperDetailExp.getSortNumber()));
    	}
    	List<PaperDetailExp> paperDetailExps = new ArrayList<PaperDetailExp>();
    	//如果没有单选
    	if(!types.contains(QuesStructType.SINGLE_ANSWER_QUESTION)){
    		buildObjectiveQuestion(paperDetailExps,questionTypeNums,QuesStructType.SINGLE_ANSWER_QUESTION);
    	}else{
    		addObjectiveQuestion(paperDetailExps,objectiveDetails, questionTypeNums, QuesStructType.SINGLE_ANSWER_QUESTION);
    	}
    	//如果没有多选
    	if(!types.contains(QuesStructType.MULTIPLE_ANSWER_QUESTION)){
    		buildObjectiveQuestion(paperDetailExps,questionTypeNums,QuesStructType.MULTIPLE_ANSWER_QUESTION);
    	}else{
    		addObjectiveQuestion(paperDetailExps, objectiveDetails, questionTypeNums, QuesStructType.MULTIPLE_ANSWER_QUESTION);
    	}
    	//如果没有判断
    	if(!types.contains(QuesStructType.BOOL_ANSWER_QUESTION)){
    		buildObjectiveQuestion(paperDetailExps,questionTypeNums,QuesStructType.BOOL_ANSWER_QUESTION);
    	}else{
    		addObjectiveQuestion(paperDetailExps, objectiveDetails, questionTypeNums, QuesStructType.BOOL_ANSWER_QUESTION);
    	}
    	return paperDetailExps;
	}
    
    /**
     * 根据数量和类型构建客观题
     * @param questionTypeNums
     * @param quesStructType
     * @return
     */
    private void buildObjectiveQuestion(List<PaperDetailExp> paperDetailExps,List<QuestionTypeNum> questionTypeNums,QuesStructType quesStructType){
    	//1.得到需要的题量
    	int quantity = getQuantity(questionTypeNums,quesStructType);
		//2.构建题目
		PaperDetailExp paperDetailExp = new PaperDetailExp();
    	List<PaperDetailUnitExp> paperDetailUnits  = new ArrayList<PaperDetailUnitExp>();
		for(int i = 0;i<quantity;i++){
			paperDetailUnits.add(new PaperDetailUnitExp(quesStructType,i+1));
		}
		paperDetailExp.setPaperDetailUnits(paperDetailUnits);
		paperDetailExp.setNumber(Integer.parseInt(quesStructType.getId()+""));
		paperDetailExps.add(paperDetailExp);
    }
    
    /**
     * 补齐需要的客观题
     * @param objectiveDetails
     * @param questionTypeNums
     */
    private void addObjectiveQuestion(List<PaperDetailExp> paperDetailExps,List<PaperDetailExp> objectiveDetails,List<QuestionTypeNum> questionTypeNums,QuesStructType quesStructType){
    	//得到当前题量
    	int currentSize = 0;
    	/**
    	 * 构建新的大题
    	 * 遍历所有的客观大题，如果是符合的类型，就添加该大题
    	 * 该大题中，如果有套题，要将该套题下的小题拆出来，添加到大题中，并删除原套题
    	 */
		for(PaperDetailExp paperDetailExp:objectiveDetails){
    		if(paperDetailExp.getSortNumber() == quesStructType.getId()){
    			PaperDetailExp newPaperDetailExp = new PaperDetailExp();
    			List<PaperDetailUnitExp> newPaperDetailUnits = new ArrayList<PaperDetailUnitExp>();
    			//得到大题下的所有小题
        		List<PaperDetailUnitExp> paperDetailUnits = paperDetailExp.getPaperDetailUnits();
        		for(PaperDetailUnitExp paperDetailUnitExp:paperDetailUnits){
        			if(paperDetailUnitExp.getQuestionType() != QuesStructType.NESTED_ANSWER_QUESTION){
        				newPaperDetailUnits.add(paperDetailUnitExp);
    					currentSize += 1;
    				}else{
    					List<Question> subQuestions = paperDetailUnitExp.getQuestion().getSubQuestions();
    					if(subQuestions!=null&&subQuestions.size()>0){
    						for(int i = 0;i<subQuestions.size();i++){
        						newPaperDetailUnits.add(new PaperDetailUnitExp(quesStructType,subQuestions.get(i)));
        					}
        					currentSize += subQuestions.size();
    					}
    				}
        		}
        		//设置小题
        		newPaperDetailExp.setPaperDetailUnits(newPaperDetailUnits);
    			//设置大题题号
        		newPaperDetailExp.setNumber(Integer.parseInt(quesStructType.getId()+""));
        		newPaperDetailExp.setSortNumber(paperDetailExp.getSortNumber());
    			paperDetailExps.add(newPaperDetailExp);
    		}
    	}
		
		//刷一遍number
		int num = 0;
		for(int i = 0;i<paperDetailExps.size();i++){
			if(paperDetailExps.get(i).getSortNumber() == quesStructType.getId()){
				List<PaperDetailUnitExp> exps = paperDetailExps.get(i).getPaperDetailUnits();
				for(int j = 0;j<exps.size();j++){
					num++;
					exps.get(j).setNumber(num);
				}
			}
		}
		
		//计算差额、补齐试题
		int quantity = getQuantity(questionTypeNums,quesStructType)-currentSize;
		PaperDetailExp paperDetailExp = new PaperDetailExp();
		List<PaperDetailUnitExp> paperDetailUnits = new ArrayList<PaperDetailUnitExp>();
		for(int i = 0;i<quantity;i++){
			int number = currentSize+(i+1);
			paperDetailUnits.add(new PaperDetailUnitExp(quesStructType,number));
		}
		paperDetailExp.setPaperDetailUnits(paperDetailUnits);
		//设置大题题号
		paperDetailExp.setNumber(Integer.parseInt(quesStructType.getId()+""));
		paperDetailExps.add(paperDetailExp);
    }
    
    //根据类型得到需要的题量
    private Integer getQuantity(List<QuestionTypeNum> questionTypeNums,QuesStructType quesStructType){
    	int quantity = 0;
		for(QuestionTypeNum typeNum:questionTypeNums){
			if(typeNum.getQuestionType() == quesStructType){
				quantity = typeNum.getQuantity();
			}
		}
		return quantity;
    }
    /**
     * 得到试卷中所有客观大题
     * 与试题类型枚举:QuesStructType中的ID对应
     * @param paperExp
     * @return
     */
    public List<PaperDetailExp> getAllObjectiveDetails(PaperExp paperExp){
    	List<PaperDetailExp> objectiveDetails = new ArrayList<PaperDetailExp>();
    	for(PaperDetailExp paperDetailExp:paperExp.getPaperDetails()){
    		if(paperDetailExp.getSortNumber()==1L
    				||paperDetailExp.getSortNumber()==2L
    					||paperDetailExp.getSortNumber()==3L){
    			objectiveDetails.add(paperDetailExp);
    		}
    	}
    	return objectiveDetails;
    }
    /**
     * 得到所有主观大题
     * 与试题类型枚举:QuesStructType中的ID对应
     * @param paperExp
     * @return
     */
    public List<PaperDetailExp> getAllSubjectiveDetails(PaperExp paperExp){
    	List<PaperDetailExp> subjectiveDetails = new ArrayList<PaperDetailExp>();
    	for(PaperDetailExp paperDetailExp:paperExp.getPaperDetails()){
    		if(paperDetailExp.getSortNumber()==4L
    				||paperDetailExp.getSortNumber()==5L
    					||paperDetailExp.getSortNumber()==6L){
    			subjectiveDetails.add(paperDetailExp);
    		}
    	}
    	return subjectiveDetails;
    }
	/**
     * 检查客观题数量是否小于试卷结构导出设置的数量
     * @param paperExp
     * @param questionTypeNums
     * @return
     */
    protected void checkObjectiveDetailsNum(PaperExp paperExp,List<QuestionTypeNum> questionTypeNums) {
       List<PaperDetailExp> objectiveDetails = getAllObjectiveDetails(paperExp);
	   for(QuestionTypeNum typeNum:questionTypeNums){
		   List<PaperDetailUnitExp> paperDetailUnits = new ArrayList<PaperDetailUnitExp>();
		   for(PaperDetailExp paperDetailExp:objectiveDetails){
			   if(paperDetailExp.getSortNumber() == typeNum.getQuestionType().getId()){
				   paperDetailUnits.addAll(paperDetailExp.getPaperDetailUnits());
			   }
		   }
		   int currentSize = 0; 
		   for(PaperDetailUnitExp paperDetailUnitExp:paperDetailUnits){
			   if(paperDetailUnitExp.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION){
				   currentSize += paperDetailUnitExp.getQuestion().getSubQuestions().size();
			   }else{
				   currentSize++;
			   }
		   }
		   if(currentSize>typeNum.getQuantity()){
			   throw new RuntimeException("试卷中"+typeNum.getQuestionType().getName()+"题题量超出设定范围"
								   		+ ",当前试卷题量:"+currentSize
								   		+ ",设定题量:"+typeNum.getQuantity()
								   		+ ",请核实后重新设置导出规则！");
		   }
	   }
    }
    /**
     * 获取当前试卷下所有试题WordPkg
     * @param id
     * @return
     */
    protected List<WordprocessingMLPackage> getPkgList(String id){
        Paper paper = paperRepo.findOne(id);
        List<WordprocessingMLPackage> wordMLPackages = paperDetailUnitRepo.findByPaperOrderByNumber(paper)
                .stream().map(PaperDetailUnit::getQuestion).collect(Collectors.toList())
                .stream().map(question -> getPkgObj(question)).collect(Collectors.toList());
        return wordMLPackages;
    }

    private WordprocessingMLPackage getPkgObj(Question question){
		String pkgPathId = question.getQuesPkgPathId();
		byte[] pkgByte = quesPkgPathRepo.findFirstById(pkgPathId).getQuesPkg();
		return DocxProcessUtil.getPkg(pkgByte);
	}
    
    /**
     * 筛选出试卷类型
     * @param extractConfig
     * @param paperId
     * @return
     */
    private String findPaperType(ExtractConfig extractConfig,String paperId){
    	String paperType = "";
    	Map<String,String> paperIdMap = extractConfig.getFinishedPaperIdMap();
    	Set<Entry<String,String>> entry = paperIdMap.entrySet();
		Iterator<Entry<String,String>> iterator = entry.iterator();
		while(iterator.hasNext()){
			Entry<String,String> next = iterator.next();
			if(paperId.equals(next.getValue())){
				paperType = next.getKey();
				break;
			}
		}
		return paperType;
    }
    
}

