package com.qmth.cqb.paper.service.impl;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;







import com.qmth.cqb.paper.dao.CoursePropertyRepo;
import com.qmth.cqb.paper.dto.CoursePropertyDto;
import com.qmth.cqb.paper.model.CourseProperty;
import com.qmth.cqb.paper.service.CoursePropertyService;

/**
 * @describle 课程属性 coursePropertyServiceImpl
 * @author weiwenhai
 * @date   2017.11.2
 */
@Service("coursePropertyService")
public class CoursePropertyServiceImpl implements CoursePropertyService{
	
	@Autowired
	private CoursePropertyRepo coursePropertyRepo;

	@Override
	public List<CourseProperty> findAllByOrgId(Long orgId) {
		List<CourseProperty> courseProperties = coursePropertyRepo.findByOrgId(orgId);
		return courseProperties;
	}

	@Override
	public void saveCourseProperty(CoursePropertyDto coursePropertyDto) throws Exception {
		coursePropertyDto.setEnable("1");
		CourseProperty courseProperty = coursePropertyRepo.findByNameAndOrgId(coursePropertyDto.getName(), coursePropertyDto.getOrgId());
		if(courseProperty != null){
			throw new Exception("课程属性名已经存在");
		}
		courseProperty = new CourseProperty(coursePropertyDto);
		coursePropertyRepo.save(courseProperty);
	}

	@Override
	public Page<CourseProperty> findAllByOrg(CoursePropertyDto coursePropertyDto, Pageable pageable) {
		Specification<CourseProperty> specification = getSpecification(coursePropertyDto);
		Page<CourseProperty> coursePropertyies = coursePropertyRepo.findAll(specification, pageable);
		return coursePropertyies;
	}

	private Specification<CourseProperty> getSpecification(CoursePropertyDto coursePropertyDto) {
		CourseProperty courseProperty = new CourseProperty(coursePropertyDto);
		Specification<CourseProperty> specification = (root, query, cb) -> {
		    List<Predicate> predicates = new ArrayList<>();
		    if(!StringUtils.isEmpty(courseProperty.getOrgId())){
		    	predicates.add(cb.equal(root.get("orgId"),courseProperty.getOrgId()));
		    }
		    if(!StringUtils.isEmpty(courseProperty.getName())){
		    	predicates.add(cb.like(root.get("name"),"%"+courseProperty.getName()+"%"));
		    }
		    if(!StringUtils.isEmpty(courseProperty.getCourseId())){
		    	predicates.add(cb.equal(root.get("courseId"),courseProperty.getCourseId()));
		    }
		    if(!StringUtils.isEmpty(courseProperty.getEnable())){
		    	predicates.add(cb.equal(root.get("enable"),courseProperty.getEnable()));
		    }
		    return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}

	@Override
	public void openCourseProperty(Long id, Long orgId) {
		CourseProperty courseProperty = coursePropertyRepo.findOne(id);
		courseProperty.setEnable(true);
		coursePropertyRepo.save(courseProperty);
	}

	@Override
	public void closeCourseProperty(Long id, Long orgId) {
		CourseProperty courseProperty = coursePropertyRepo.findOne(id);
		courseProperty.setEnable(false);
		coursePropertyRepo.save(courseProperty);
	}

	@Override
	public List<CourseProperty> findAllByCourseId(Long courseId) {
		List<CourseProperty> courseProperties = coursePropertyRepo.findByCourseId(courseId);
		return courseProperties;
	}
}
