package com.qmth.cqb.paper.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.paper.service.PaperDetailService;
import com.qmth.cqb.paper.service.PaperDetailUnitService;
import com.qmth.cqb.paper.service.QuesTypeNameService;

/**
 * @author  	chenken
 * @date    	2017年9月13日 上午11:32:33
 * @company 	QMTH
 * @description PaperDetailServiceImpl.java
 */
@Service("paperDetailService")
public class PaperDetailServiceImpl implements PaperDetailService{

    @Autowired
    PaperDetailRepo paperDetailRepo;

    @Autowired
    PaperDetailUnitService unitService;

    @Autowired
    PaperRepo paperRepo;
    
    @Autowired
    QuesTypeNameService quesTypeNameService;


    @Autowired
    PaperDetailUnitRepo paperDetailUnitRepo;

    /**
     * 根据Id获得对应所有小题
     * 
     * @param id
     * @return
     */
    public List<PaperDetailUnit> getUnitsByPaperDetailId(String id) {
        return unitService.getUnitsByPaperDetail(paperDetailRepo.findOne(id));
    }

    /**
     * 按ID查询大题
     * 
     * @param id
     * @return
     */
    public PaperDetail findById(String id) {
        return paperDetailRepo.findOne(id);
    }

    /**
     * 保存大题
     * 
     * @param pd
     * @return
     */
    public PaperDetail savePaperDetail(PaperDetail pd, String paperId, AccessUser user) {
        Paper paper = paperRepo.findOne(paperId);
        paper.setLastModifyName(user.getName());
        paper = paperRepo.save(paper);
        pd.setPaper(paper);
        PaperDetail paperDetail = paperDetailRepo.save(pd);
        List<PaperDetailUnit> paperDetailUnits = paperDetailUnitRepo.findByPaperDetailOrderByNumber(paperDetail);
        quesTypeNameService.saveQuesTypeName(paperDetailUnits);
        return paperDetail;
    }

    /**
     * 删除大题
     * 
     * @param id
     * @return
     */
    public void deletePaperDetail(String id) {
        paperDetailRepo.delete(id);
    }

    /**
     * 查询同一个试卷对象对应大题
     * 
     * @param paper
     * @return
     */
    public List<PaperDetail> getPaperDetailsByPaper(Paper paper) {
        return paperDetailRepo.findByPaperOrderByNumber(paper);
    }

    /**
     * 删除试卷大题
     * 
     * @param paper
     */
    public void deletePaperDetailsByPaper(Paper paper) {
        List<PaperDetail> details = getPaperDetailsByPaper(paper);
        unitService.deleteUnitsByPaperDetails(details);
        paperDetailRepo.delete(details);
    }

    /**
     * 批量删除试卷大题
     * 
     * @param papers
     */
    public void deletePaperDetailsByPapers(List<Paper> papers) {
        for (Paper paper : papers) {
            List<PaperDetail> details = getPaperDetailsByPaper(paper);
            unitService.deleteUnitsByPaperDetails(details);
            paperDetailRepo.delete(details);
        }
    }
}

