package com.qmth.cqb.paper.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

import com.qmth.cqb.paper.assemble.PropertyAssembler;
import com.qmth.cqb.paper.dao.PropertyRepo;
import com.qmth.cqb.paper.dto.PropertyDto;
import com.qmth.cqb.paper.model.Property;
import com.qmth.cqb.paper.service.PropertyService;
/**
 * @describle 课程属性 coursePropertyServiceImpl
 * @author weiwenhai
 * @date   2017.11.8
 */
@Service("propertyService")
public class PropertyServiceImpl implements PropertyService{
	
	@Autowired
	private PropertyRepo propertyRepo;

	@Override
	public void saveProperty(List<PropertyDto> propertyDtos, AccessUser user) {
		PropertyAssembler propertyAssembler = new PropertyAssembler();
		for(int i=0;i<propertyDtos.size();i++){
			//取到父类节点对象
			Property property = propertyAssembler.toProperty(propertyDtos.get(i));
			property.setNumber(i+1);
			property.setOrgId(user.getRootOrgId());
			property = propertyRepo.save(property);
			List<Property> properties = propertyAssembler.toPropertySon(propertyDtos.get(i));
			if(properties != null && properties.size()>0){
				for(int j=0;j<properties.size();j++){
					properties.get(j).setParentId(property.getId());
					properties.get(j).setNumber(j+1);
				}
				propertyRepo.save(properties);
			}
		}
	}

	@Override
	public List<PropertyDto> propertyDtos(Long coursePropertyId, AccessUser user) {
		List<PropertyDto> propertyDtos = new ArrayList<PropertyDto>();
		//查询所有父节点对象
		List<Property> propertieParents = propertyRepo.findByOrgIdAndCoursePropertyIdAndParentIdOrderByNumber(user.getRootOrgId(), coursePropertyId, 0l);
		if(propertieParents != null && propertieParents.size()>0){
			for(Property property:propertieParents){
				PropertyDto propertyDtoParent = new PropertyDto(property);
				//查询所有子节点对象
				List<Property> propertiesSons = propertyRepo.findByOrgIdAndCoursePropertyIdAndParentIdOrderByNumber(user.getRootOrgId(), coursePropertyId, property.getId());
				if(propertiesSons != null && propertiesSons.size()>0){
					List<PropertyDto> propertyDtosSons = new ArrayList<PropertyDto>();
					for(Property property2:propertiesSons){
						PropertyDto propertyDto2 = new PropertyDto(property2);
						propertyDtosSons.add(propertyDto2);
					}
					propertyDtoParent.setPropertyDtos(propertyDtosSons);
				}
				propertyDtos.add(propertyDtoParent);
			}
		}
		return propertyDtos;
	}

	@Override
	public void saveProperty(Property property, AccessUser user) {
		if(property.getId() == null){
			Integer number = 0;
			List<Property> propertieParents = propertyRepo.findByOrgIdAndCoursePropertyIdAndParentIdOrderByNumber(user.getRootOrgId(), property.getCoursePropertyId(), property.getParentId());
			if(propertieParents != null && propertieParents.size()>0){
				number = propertieParents.get(propertieParents.size()-1).getNumber();
			}
			property.setNumber(number+1);
		}
		property.setOrgId(user.getRootOrgId());
		propertyRepo.save(property);
	}

	@Override
	public void deleteProperty(Long propertyId, Long coursePropertyId, AccessUser user) {
		//查询子节点对象
		List<Property> properties = propertyRepo.findByOrgIdAndCoursePropertyIdAndParentIdOrderByNumber(user.getRootOrgId(), coursePropertyId, propertyId);
		if(properties != null && properties.size()>0){
			for(Property property:properties){
				propertyRepo.delete(property);
			}
		}
		propertyRepo.delete(propertyId);
	}

	@Override
	public List<Property> findAll(Long coursePropertyId, AccessUser user) {
		List<PropertyDto> propertyDtos = propertyDtos(coursePropertyId, user);
		List<Property> properties = new ArrayList<Property>();
		if(propertyDtos != null && propertyDtos.size() >0){
			for(PropertyDto propertyDto:propertyDtos){
				Property property = new Property(propertyDto);
				properties.add(property);
				if(propertyDto.getPropertyDtos() != null && propertyDto.getPropertyDtos().size() > 0){
					for(PropertyDto propertyDtoSon:propertyDto.getPropertyDtos()){
						Property propertySon = new Property(propertyDtoSon);
						properties.add(propertySon);
					}
				}
			}
		}
		return properties;
	}

	@Override
	public List<Property> findPropertySons(Property property) {
		List<Property> properySonsList = propertyRepo.findByParentIdOrderByNumber(property.getId());
		return properySonsList;
	}

}
