package com.qmth.cqb.question.service.impl;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import main.java.com.UpYun;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

import com.qmth.cqb.question.dao.QuesRepo;
import com.qmth.cqb.question.dao.QuestionAudioRepo;
import com.qmth.cqb.question.model.QuesOption;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.question.model.QuestionAudio;
import com.qmth.cqb.question.service.QuestionAudioService;

/**
 * @author  	chenken
 * @date    	2017年8月1日 上午11:27:33
 * @company 	QMTH
 * @description QuestionRadioServiceImpl.java
 */
@Service("questionAudioService")
public class QuestionAudioServiceImpl implements QuestionAudioService{

	@Autowired
	private QuestionAudioRepo questionAudioRepo;
	
	@Autowired
	private QuesRepo quesRepo;
	
	@Value("${upyun.bucketName}")
	protected String bucketName;
	
	@Value("${upyun.userName}")
	protected String userName;
	
	@Value("${upyun.password}")
	protected String password;
	
	@Override
	public void saveQuestionAudio(QuestionAudio questionAudio,AccessUser accessUser) {
		QuestionAudio questionAudio2 = findByQuestionIdAndFileName(questionAudio.getQuestionId(),questionAudio.getFileName());
    	if(questionAudio2 != null){
    		UpYun upYun = new UpYun(bucketName, userName, password);
    		upYun.deleteFile(questionAudio2.getFileUrl());
    		questionAudioRepo.delete(questionAudio2);
    	}
		questionAudio.setCreateTime(new Date());
		questionAudio.setCreateUser(accessUser.getName());
		questionAudioRepo.save(questionAudio);
	}

	@Override
	public List<QuestionAudio> findQuestionAudiosByQuestionId(String questionId) {
		Assert.hasLength(questionId, "questionId不能为空");
		return questionAudioRepo.findByQuestionId(questionId);
	}

	@Override
	public QuestionAudio findByQuestionIdAndFileName(String questionId,String fileName) {
		if(StringUtils.isBlank(questionId)||StringUtils.isBlank(fileName)){
			return null;
		}
		return questionAudioRepo.findByQuestionIdAndFileName(questionId,fileName);
	}

	@Override
	public void deleteAudioNotInQuestion(Question question) {
		if(question.getHasAudio()!=null&&question.getHasAudio()){
			List<QuestionAudio> questionAudios = questionAudioRepo.findByQuestionId(question.getId());
			StringBuffer buffer = new StringBuffer();
			buffer.append(question.getQuesBody());
			if(question.getQuesOptions()!=null&&question.getQuesOptions().size()>0){
				for(QuesOption option:question.getQuesOptions()){
					buffer.append(option.getOptionBody());
				}
			}
			String questionBodyAndOptionStr = buffer.toString();
			Iterator<QuestionAudio> audioIterator = questionAudios.iterator();
			while(audioIterator.hasNext()){
				QuestionAudio audio = audioIterator.next();
				if(!questionBodyAndOptionStr.contains(audio.getFileName())){
					UpYun upYun = new UpYun(bucketName, userName, password);
					upYun.deleteFile(audio.getFileUrl());
					questionAudioRepo.delete(audio);
				}
			}
		}
	}

	@Override
	public QuestionAudio findAudioById(String id) {
		if(StringUtils.isNotBlank(id)){
			return questionAudioRepo.findOne(id);
		}
		return null;
	}

}

